/**
 * Dependencies
 */
var request = require("request");
var config = require("./config");
var btoa = require("btoa");

function General() {
	/**
	 * Variables
	 */
	this.privateKey = config.get("privateKey");
	this.Authorization = `{email}:${this.privateKey}`;
	this.appId = config.get("appId");
	this.ContentType = "application/json; charset=utf-8";

	/**
	 * Get Authorization URL for new account
	 * @param  {String} accountType
	 * @param  {String} userEmail   Unique identifier for the user
	 * @return {String}             URL
	 */
	this.getUrl = (accountType, userEmail) => {
		return new Promise((resolve, reject) => {
			let url = config.get(`Endpoints.${accountType}`) + "/authorize"

			request.post({
				headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
				url,
			}, function (error, response, body) {
				if (error) { return reject(error) }
				if (response.statusCode != 200) { return reject(response) }
				resolve(JSON.parse(body).authorizeUrl);
			});
		})
	}

	/**
	 * Create an account after authorization
	 * @param  {String} accountType
	 * @param  {String} userEmail   Unique identifier for the user
	 * @param  {String} queryString 	Received QueryString from redirect
	 * @return {Account}             	The newly created Account
	 */
	this.createAccount = (accountType, userEmail, queryString) => {
		return new Promise((resolve, reject) => {
			let url = config.get(`Endpoints.${accountType}`) + "/token"

			request.post({
				headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
				url,
			}, function (error, response, body) {
				if (error) { return reject(error) }
				if (response.statusCode != 201) { return reject(response) }
				resolve(JSON.parse(body));
			});
		})
	}

	/**
	 * Delete an existing Account
	 * @param  {String} accountType
	 * @param  {String} userEmail   Unique identifier for the user
	 * @param  {String} accountId
	 */
	this.deleteAccount = (accountType, userEmail, accountId) => {
		return new Promise((resolve, reject) => {
			let url = config.get(`Endpoints.${accountType}`) + `/accounts/${accountId}`

			request.delete({
				headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
				url,
			}, function (error, response, body) {
				if (error) { return reject(error) }
				if (response.statusCode != 204) { return reject(response) }
				resolve();
			});
		})
	}

	/**
	 * Get Change from webhook token
	 * @param  {String} accountType
	 * @param  {String} token
	 * @return {Change}             The change object
	 */
	this.getChangeByToken = (accountType, token) => {
		return new Promise((resolve, reject) => {
			let url = config.get(`Endpoints.${accountType}`) + "/changes/get"

			request.post({
				headers: { Authorization: this.getAuthorization("team@fastee.im"), "Content-Type": this.ContentType, },
				url,
			}, function (error, response, body) {
				if (error) { return reject(error) }
				if (response.statusCode != 200) { return reject(response) }
				resolve(JSON.parse(body).change);
			});
		})
	}

	/**
	 * Get all acounts for user email
	 * @param  {String} userEmail
	 * @return {accounts} array of accounts
	 */
	this.listAccounts = (accountType, userEmail) => {
		return new Promise((resolve, reject) => {
			let url = config.get(`Endpoints.${accountType}`) + "/accounts";

			request.get({
				headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
				url,
			}, function (error, response, body) {
				if (error) { return reject(error) }
				if (response.statusCode != 200) { return reject(response) }
				resolve(JSON.parse(body).accounts);
			});
		})
	}

	/**
	 * Get the final Authorization header for user email
	 * @param  {String} userEmail
	 * @return {String}           Final Authorization header for the request
	 */
	this.getAuthorization = (userEmail) => {
		return "Basic " + btoa(this.Authorization.replace("{email}", userEmail))
	}
}

/**
 * GET Snapshot
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} snapshotId
 * @return {Snapshot} 
 */
this.getSnapshot = function (accountType, userEmail, snapshotId) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/snapshots/${snapshotId}`

		request.get({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 200) { return reject(response) }
			resolve(JSON.parse(body));
		});
	})
}

/**
 * GET Snapshot Body
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} snapshotId
 * @return STREAM 
 */
this.getSnapshotBody = function (accountType, userEmail, snapshotId) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/snapshots/${snapshotId}/content`

		request.get({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 200) { return reject(response) }
			resolve(body);
		});
	})
}

module.exports = General;
