/**
 * Dependencies
 */
var request = require("request");
var config = require("../config");
var General = require("../general");

function Email() {
	General.call(this);
}

/**
 * GET List mailboxes (like Inbox, Sent etc)
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} accountId
 * @param  {String} pager			(optional)
 * @param  {String} folderId		(optional)
 * @return {Object}					{ folder: Folder[], pager: string, }
 */
Email.prototype.listMailboxes = function (accountType, userEmail, accountId, pager, folderId) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/${accountId}/folders`

		request.get({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url,
			qs: {
				pager,
				folderId
			}
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 200) { return reject(response) }
			resolve(JSON.parse(body));
		});
	})
}

/**
 * GET List emails
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} accountId
 * @param  {String} pager			(optional)
 * @param  {String} folderId		(optional)
 * @param  {String} search			(optional)
 * @param  {String} filter			(optional) CreatedOn, From, To (set of email address, separated by comma)
 * 											   Example: from=“mike@hubx.io,lizy@fastee.im” createdon>"2016-07-25T20:00:00Z"
 * @param  {Int} 	top				(optional)
 * @return {Object}					{ mails: Mail[], pager: string, }
 */
Email.prototype.listEmails = function (accountType, userEmail, accountId, folderId, pager, search, filter, top) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/${accountId}/mails`;

		request.get({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url,
			qs: {
				pager,
				folderId,
				search,
				filter,
				top
			}
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 200) { return reject(response) }
			resolve(JSON.parse(body));
		});
	})
}

/**
 * GET Mail Body
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} accountId
 * @param  {String} mailId
 * @return STREAM
 */
Email.prototype.getMailBody = function (accountType, userEmail, accountId, mailId) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/${accountId}/mails/${mailId}/content`;

		request.get({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 200) { return reject(response) }
			resolve(body);
		});
	})
}

/**
 * GET Attachment Content
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} accountId
 * @param  {String} mailId
 * @param  {String} attachmentId
 * @return STREAM
 */
Email.prototype.getAttachmentContent = function (accountType, userEmail, accountId, mailId, attachmentId) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/${accountId}/mails/${mailId}/attachments/${attachmentId}/content`;

		request.get({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 200) { return reject(response) }
			resolve(body);
		});
	})
}

/**
 * GET List Mail Attachments
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} accountId
 * @param  {String} pager		(optional)
 * @param  {String} mailId		(optional)
 * @return {Object}				{ pager:<next-page-reference>,  files: [ Array of Attachment object ] }
 */
Email.prototype.listMailAttachments = function (accountType, userEmail, accountId, mailId, pager) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/${accountId}/mails/${mailId}/attachments?pager=${pager}`;

		request.get({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url,
			qs: {
				pager,
				folderId,
				search,
				filter,
			}
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 200) { return reject(response) }
			resolve(JSON.parse(body));
		});
	})
}


/**
 * PUT Create a Snapshot
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} accountId
 * @param  {String} mailId
 * @return {Snapshot}             The newly created Snapshot
 */
Email.prototype.createSnapshot = function (accountType, userEmail, accountId, mailId) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/${accountId}/mails/${mailId}/snapshot`;

		request.put({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url,
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 201) { return reject(response) }
			resolve(JSON.parse(body));
		});
	})
}

module.exports = Email
