/**
 * Dependencies
 */
var request = require("request");
var config = require("../config");
var General = require("../general");

function Storage() {
    General.call(this);
}

/**
 * GET List Folders
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} accountId
 * @param  {String} pager			(optional)
 * @param  {String} folderId		(optional)
 * @param  {String} search			(optional)
 * @param  {Int} 	top				(optional)
 * @return {Object}					folders: [ Array of Folder object ]
 */
Storage.prototype.listFolders = function (accountType, userEmail, accountId, pager, folderId, search, top) {
    return new Promise((resolve, reject) => {
        let url = config.get(`Endpoints.${accountType}`) + `/${accountId}/folders`;

        request.get({
            headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
            url,
            qs: {
                pager,
                folderId,
                search,
                top
            }
        }, function (error, response, body) {
            if (error) { return reject(error) }
            if (response.statusCode != 200) { return reject(response) }
            resolve(JSON.parse(body));
        });
    })
}

/**
 * GET List Files
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} accountId
 * @param  {String} pager			(optional)
 * @param  {String} folderId		(optional)
 * @param  {String} search			(optional)
 * @param  {String} filter			(optional) ModifiedOn (time), From (set of email address, separated by comma)
 * 											   Example: from=“mike@hubx.io,lizy@fastee.im” ModifiedOn >"2016-07-25T20:00:00Z"
 * @param  {Int} 	top				(optional)
 * @return {Object}					folders: [ Array of Folder object ]
 */
Storage.prototype.listFiles = function (accountType, userEmail, accountId, pager, folderId, search, filter, top) {
    return new Promise((resolve, reject) => {
        let url = config.get(`Endpoints.${accountType}`) + `/${accountId}/files`;

        request.get({
            headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
            url,
            qs: {
                pager,
                folderId,
                search,
                filter,
                top
            }
        }, function (error, response, body) {
            if (error) { return reject(error) }
            if (response.statusCode != 200) { return reject(response) }
            resolve(JSON.parse(body));
        });
    })
}

/**
 * GET File Content
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} accountId
 * @param  {String} fileId
 * @return STREAM
 */
Storage.prototype.getFileContent = function (accountType, userEmail, accountId, fileId) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/${accountId}/files/${fileId}/content`;

		request.get({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 200) { return reject(response) }
			resolve(body);
		});
	})
}

/**
 * PUT Create a Snapshot
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} accountId
 * @param  {String} fileId
 * @return {Snapshot}             The newly created Snapshot
 */
Storage.prototype.createSnapshot = function (accountType, userEmail, accountId, fileId) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/${accountId}/files/${fileId}/snapshot`;

		request.put({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url,
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 201) { return reject(response) }
			resolve(JSON.parse(body));
		});
	})
}

/**
 * GET List File Snapshots
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} fileId          
 * @return {Object}					{ snapshots: [ Array of Snapshot object ] }
 */
Storage.prototype.listFiles = function (accountType, userEmail, fileId) {
    return new Promise((resolve, reject) => {
        let url = config.get(`Endpoints.${accountType}`) + `/snapshots`;

        request.get({
            headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
            url,
            qs: {
                fileId
            }
        }, function (error, response, body) {
            if (error) { return reject(error) }
            if (response.statusCode != 200) { return reject(response) }
            resolve(JSON.parse(body));
        });
    })
}

/**
 * GET Thumbnail Content
 * @param  {String} accountType
 * @param  {String} userEmail
 * @param  {String} snapshotId
 * @param  {String} fileId
 * @return STREAM
 */
Storage.prototype.getFileContent = function (accountType, userEmail, snapshotId) {
	return new Promise((resolve, reject) => {
		let url = config.get(`Endpoints.${accountType}`) + `/snapshots/${snapshotId}/thumbnail`;

		request.get({
			headers: { Authorization: this.getAuthorization(userEmail), "Content-Type": this.ContentType, },
			url
		}, function (error, response, body) {
			if (error) { return reject(error) }
			if (response.statusCode != 200) { return reject(response) }
			resolve(body);
		});
	})
}

module.exports = Storage